
import java.io.*;
import java.security.*;
import jonelo.jacksum.*;
import jonelo.jacksum.algorithm.*;
import java.nio.channels.*;


 public class Makefile
 {

    public static void copyFile(File in, File out)   
    {
        FileChannel inChannel=null;
        FileChannel outChannel=null;
        try {
	    inChannel  =  new  FileInputStream(in).getChannel();
	    outChannel = new   FileOutputStream(out).getChannel();
            inChannel.transferTo(0, inChannel.size(),outChannel);
        } 
        catch (Exception e) {
	    System.out.println("Exception : " + e);
	    System.exit(-1);
	    
        }
        finally 
        {
	  try
	  {
            if (inChannel != null) inChannel.close();
            if (outChannel != null) outChannel.close();
	  }
	  catch( Throwable t ) {}
        }
   }

   public static AbstractChecksum getSha1() 
   {
     AbstractChecksum checksum = null;
      try 
      {
        return JacksumAPI.getChecksumInstance("sha1");
      }
      catch (NoSuchAlgorithmException nsae) 
      {
	 System.out.println("Impossible!");
	 System.exit(-1);
	 return null;
      }
   }

   public static String checksum( String filename ) 
   {
      try {
	  AbstractChecksum checksum = getSha1();
	    checksum.readFile(filename);
      	    return checksum.getFormattedValue();
      }
      catch (IOException ioe) 
      {
           System.out.println("Exception : " + ioe);
	   System.exit(-1);
	   return null;
      }
   }

   public static String checksum( byte[] b ) 
   {
        AbstractChecksum checksum = getSha1();
          checksum.reset();
          checksum.update(b);
	  return checksum.getFormattedValue();
   }

    public static String makeFilename( String name , String version , String ext )
    {
        return name + "-" + version + "." + ext;
    }

    public static String genPOM( String name , String version )
    {
	return "<project>\n <modelVersion>4.0.0</modelVersion>\n <groupId>perfsonar</groupId>\n <artifactId>" + name +"</artifactId>\n" +
	       "<version>" + version + "</version>\n</project>\n";
    }

    public static String genSignFile( String name , String version, String ext, String checksum)
    {
       return  checksum + " " + makeFilename(name,version,ext)  + "\n";
    }

    public static void writeFile( String file , String contents )
    {
      try
      {
        BufferedWriter out = new BufferedWriter( new FileWriter(file));
          out.write(contents);
          out.close();
      }
      catch (Exception e)
      { 
        System.err.println("Error: " + e.getMessage());
	System.exit(-1);
      }
    }
	
    public static void genFiles( String jarFile , String destPath , String name , String version )
    {
	destPath += File.separator + version;
	if(!new File(destPath).mkdir())
        {
	  System.out.println("Error: Could not create directory " + version + " at path " + destPath);
	  System.exit(-1);
	}
	destPath += File.separator;
	copyFile( new File(jarFile) , new File( destPath + makeFilename(name,version,"jar")) );
	writeFile( destPath + makeFilename(name,version,"jar.sha1") , genSignFile(name,version,"jar", checksum(jarFile)) );
	writeFile( destPath + makeFilename(name,version,"pom") ,  genPOM(name,version) );
	writeFile( destPath + makeFilename(name,version,"pom.sha1") , genSignFile(name,version,"pom", checksum(genPOM(name,version).getBytes() )));
    } 

    public static void main( String[] args )
    {
	if( args.length != 4 ) 
	{
	   System.out.println("Makefile <path to jar file> <destination path> <new jar file name> <new jar file version>\n"+
			      "<destination path> = some_path/jar-repository/perfsonar/name_of_the_release\n" + 
			      "<new jar file name> = e.g web-admin\n<new jar file version> = eg. 0.9.1b");
	   System.exit(-1);
	}
	else genFiles(args[0],args[1],args[2],args[3]);
    }
 };
